@@ -12,14 +12,14 @@ fi
 
 FIRST=YES
 for LIB in rt xnet; do
-	if ${CC} -l${LIB} posix-l.c 2>/dev/null; then
+	if ${CC} ${CFLAGS} -l${LIB} posix-l.c 2>/dev/null; then
 		if [ ${FIRST} = "NO" ]; then
 			printf " ";
 		fi
 		printf "%s" "-l${LIB}";
 		FIRST=NO;
 	else
-		echo "WARNING: POSIX violation: make's CC doesn't understand -l${LIB}" 1>&2
+		echo "WARNING: POSIX violation: ${CC} does not understand -l${LIB}" 1>&2
 	fi
 	rm -f a.out
 done
